close all
clear
clc

addpath('/opt/openEMS/share/openEMS/matlab');
addpath('/opt/openEMS/share/CSXCAD/matlab');
addpath('/opt/openEMS/share/hyp2mat/matlab');
addpath('/opt/openEMS/share/CTB/matlab');
physical_constants; #import physical_constants

###############################
## PREPARE SIMULATION MODEL
###############################

#variables

fmin = 2e9;
fmax = 21e9;
impedance = 50;
eps_r = 4.5; #FR-4 permability
tan_loss = 0.02; #FR-4 losses (as tangens)
kap = tan_loss*2*pi*((fmin*fmax)^0.5)*EPS0*eps_r; #FR-4 losses (as kappa)
res = c0 / (fmax*sqrt(eps_r)) / 50;
res_f = res / 4;

subst_h = 1e-3;
boxsize_x = 10e-3;
boxsize_y = 9e-3;
boxsize_z = 7e-3;

port_pos_x = -1.3e-3 / 2;
feedline_w = 0.2e-3;
lumpc_l = 0.5e-3;
lumpc_w = 0.25e-3;
lumpc_h = 0.15e-3;
c_ref = 0.687e-12;

#initiate and setup simulation structures and parameters
FDTD = InitFDTD();
FDTD = SetGaussExcite(FDTD, fmin, fmax);
BC = {'PML_8' 'PML_8' 'PML_8' 'PML_8' 'PEC' 'PEC'};
FDTD = SetBoundaryCond(FDTD, BC);
CSX = InitCSX();

#definitions of used materials
CSX = AddMaterial(CSX, 'FR4');
CSX = SetMaterialProperty(CSX, 'FR4', 'Epsilon', eps_r, 'Kappa', kap);
CSX = AddMetal(CSX,'Metal', 56e6, 16e-6);

#3d model import:
CSX = ImportSTL(CSX, 'Metal', 500, '/home/michal/ELEKTRONIKA/asdf_sa/LO1rev2/design/lpf_out/fc/lpf_stub2.stl');

#substrate
start = [-boxsize_x/2, -boxsize_y/2, 0];
stop = [boxsize_x/2, boxsize_y/2, subst_h];
CSX = AddBox(CSX, 'FR4', 400, start, stop);

#setup mesh
mesh.x = SmoothMeshLines([-3e-3, port_pos_x, 3e-3], res_f, 1.5);
mesh.x = SmoothMeshLines([-boxsize_x/2, mesh.x, boxsize_x/2], res, 1.5);
meshytmp = [-feedline_w/2-2/3*res_f, -feedline_w/2+1/3*res_f, feedline_w/2+2/3*res_f, feedline_w/2-1/3*res_f];
mesh.y = SmoothMeshLines([-0.003, meshytmp, 0.003], res_f, 1.5);
mesh.y = SmoothMeshLines([-boxsize_y/2, mesh.y, boxsize_y/2], res, 1.5);
mesh.z = SmoothMeshLines([0, subst_h, 2*subst_h], res_f, 1.5);
mesh.z = SmoothMeshLines([mesh.z, boxsize_z], res, 1.5);
CSX = DefineRectGrid(CSX, 1, mesh);


#add ports
start = [port_pos_x, -feedline_w/2, 0];
stop = [port_pos_x, feedline_w/2, subst_h];
[CSX,port{1}] = AddLumpedPort(CSX, 999, 1, impedance, start, stop, [0 0 1], true);


###############################
## PREPARE AND SIMULATE
###############################

#set files, paths, write data, show geometry, run the simulator
confirm_recursive_rmdir(0); #don't want asking when remomoving directory
sim_path = 'sim_path_s2';
sim_file = 'sim_file.xml';
rmdir(sim_path, 's');
mkdir(sim_path);
#save structures with data about simulation to file
WriteOpenEMS([sim_path '/' sim_file], FDTD, CSX);
#run a geometry viewer
CSXGeomPlot([sim_path '/' sim_file]);
#simulate
RunOpenEMS(sim_path, sim_file);


###############################
## PROCESS RESULTS
###############################

f = linspace(fmin, fmax, 2000);
port = calcPort(port, sim_path, f, 'RefImpedance', impedance);
z_stub = port{1}.uf.tot ./ port{1}.if.tot;
z_ref = port{2}.uf.tot ./ port{2}.if.tot;
z_nom = 1 ./ (2*pi*f*c_ref);
figure(1);
hold on;
grid on;
loglog(f/1e9, abs(z_stub), 'k-', 'LineWidth', 2);
loglog(f/1e9, abs(z_nom), 'y-', 'LineWidth', 2);

figure(2);
hold on;
grid on;
semilogx(f/1e9, (arg(z_stub).*(180/pi)), 'k-', 'LineWidth', 2);
