close all;
clear;
clc

addpath('/opt/openEMS/share/openEMS/matlab');
addpath('/opt/openEMS/share/CSXCAD/matlab');


###############################
## PREPARE SIMULATION MODEL
###############################

#constants
physical_constants; #import physical_constants
impedance = 50; #reference impedance
fmin = 2e9;  #min frquency
fmax = 6e9;  #max frquency
eps_r = 4.5; #FR-4 permability
tan_loss = 0.02; #FR-4 losses (as tangens)
kap = tan_loss*2*pi*((fmin*fmax)^0.5)*EPS0*eps_r; #FR-4 losses (as kappa)
resolution = c0 / (fmax*sqrt(eps_r)) / 60;

#structure dimensions
rod_len = 17.3/1000;
rod_len_end = 17.3/1000;
rod_width = 3/1000;
rod_thick = 1/1000;
rodcpl_width = 3/1000;
dist_pcb_thick = 1.5/1000;
box_w = 21/1000;
box_l = 50/1000;
box_h = rod_thick+dist_pcb_thick*2;
rod_s01 = 1.3/1000;
rod_s12 = 4.1/1000;
rod_s23 = 4.4/1000;


#mesh dimensions
res_coarse = c0 / fmax / 80;
res_fine = res_coarse / 4;
mesh_start_x = -box_l/2-0.01;
mesh_stop_x = box_l/2+0.01;
mesh_start_y = -box_w/2-0.01;
mesh_stop_y = box_w/2+0.01;
mesh_start_z = -dist_pcb_thick-0.008;
mesh_stop_z = rod_thick+dist_pcb_thick+0.008;

#setup FDTD
FDTD = InitFDTD('EndCriteria', 1e-5);
FDTD = SetGaussExcite(FDTD, (fmax+fmin)/2, (fmax-fmin)/2);  #finite energy excitation
BC = {'PML_8' 'PML_8' 'PML_8' 'PML_8' 'PML_8' 'PML_8'}; #boundary conditions
FDTD = SetBoundaryCond(FDTD, BC);

#setup CSX
CSX = InitCSX();

#add materials
CSX = AddMetal(CSX,'Metal');  #new metal material
CSX = AddMetal(CSX,'Metal1');  #new metal material
CSX = AddMetal(CSX,'Metal2');  #new metal material
CSX = AddMetal(CSX,'Metal3');  #new metal material
CSX = AddMetal(CSX,'Metal4');  #new metal material
CSX = AddMetal(CSX,'Metal5');  #new metal material
CSX = AddMaterial(CSX, 'FR4');
CSX = SetMaterialProperty(CSX, 'FR4', 'Epsilon', eps_r, 'Kappa', kap);

#add model
CSX = ImportSTL(CSX, 'Metal1', 600, '/home/michal/ELEKTRONIKA/asdf_sa/SIGN_CONV/design/if_filter/Interd5th_milledPCB/3d/_qe_res_t1.stl','Transform',{'Scale',1/1000});
CSX = ImportSTL(CSX, 'Metal2', 600, '/home/michal/ELEKTRONIKA/asdf_sa/SIGN_CONV/design/if_filter/Interd5th_milledPCB/3d/_qe_res_t2.stl','Transform',{'Scale',1/1000});
CSX = ImportSTL(CSX, 'Metal3', 600, '/home/michal/ELEKTRONIKA/asdf_sa/SIGN_CONV/design/if_filter/Interd5th_milledPCB/3d/_qe_res_b.stl','Transform',{'Scale',1/1000});
CSX = ImportSTL(CSX, 'Metal4', 600, '/home/michal/ELEKTRONIKA/asdf_sa/SIGN_CONV/design/if_filter/Interd5th_milledPCB/3d/cover_b.stl','Transform',{'Scale',1/1000});
CSX = ImportSTL(CSX, 'Metal5', 600, '/home/michal/ELEKTRONIKA/asdf_sa/SIGN_CONV/design/if_filter/Interd5th_milledPCB/3d/cover_t.stl','Transform',{'Scale',1/1000});
CSX = ImportSTL(CSX, 'FR4', 500, '/home/michal/ELEKTRONIKA/asdf_sa/SIGN_CONV/design/if_filter/Interd5th_milledPCB/3d/_qe_substr.stl','Transform',{'Scale',1/1000});
CSX = ImportSTL(CSX, 'FR4', 500, '/home/michal/ELEKTRONIKA/asdf_sa/SIGN_CONV/design/if_filter/Interd5th_milledPCB/3d/dist_t.stl','Transform',{'Scale',1/1000});
CSX = ImportSTL(CSX, 'FR4', 500, '/home/michal/ELEKTRONIKA/asdf_sa/SIGN_CONV/design/if_filter/Interd5th_milledPCB/3d/dist_b.stl','Transform',{'Scale',1/1000});

start = [0.011, 0.0005, 0];
stop = [0.011, -0.0005, 0.001];
CSX = AddBox(CSX, 'Metal', 400, start, stop);
#add ports
start = [rod_s23+rod_s12+rod_s01+rod_width*2.5+rodcpl_width/2-1.3/2/1000, box_w/2+0.003, 0];
stop = [rod_s23+rod_s12+rod_s01+rod_width*2.5+rodcpl_width/2+1.3/2/1000, box_w/2+0.003, 0.001];
[CSX,port{1}] = AddLumpedPort(CSX, 999, 1, impedance, start, stop, [0 0 1], true);
start = [0.011, 0, -0.0015];
stop = [0.011, 0, 0];
[CSX,port{2}] = AddLumpedPort(CSX, 999, 2, impedance, start, stop, [0 0 1]);

setup mesh
mesh.x = SmoothMeshLines([0.011,...
                          -box_l/2-0.003,...
                          -rod_width*0.5+1/3*res_fine, -rod_width*0.5-2/3*res_fine, rod_width*0.5-1/3*res_fine, rod_width*0.5+2/3*res_fine,...
                          -rod_s23-rod_width*0.5-1/3*res_fine, -rod_s23-rod_width*0.5+2/3*res_fine, -rod_s23-rod_width*1.5+1/3*res_fine, -rod_s23-rod_width*1.5-2/3*res_fine,...
                          rod_s23+rod_width*0.5+1/3*res_fine, rod_s23+rod_width*0.5-2/3*res_fine, rod_s23+rod_width*1.5-1/3*res_fine, rod_s23+rod_width*1.5+2/3*res_fine,...
                          -rod_s23-rod_s12-rod_s01-rod_width*2.5-rodcpl_width/2, rod_s23+rod_s12+rod_s01+rod_width*2.5+rodcpl_width/2,...
                          -rod_s23-rod_s12-rod_width*1.5-1/3*res_fine, -rod_s23-rod_s12-rod_width*1.5+2/3*res_fine, -rod_s23-rod_s12-rod_width*2.5+1/3*res_fine, -rod_s23-rod_s12-rod_width*2.5-2/3*res_fine,...
                          rod_s23+rod_s12+rod_width*1.5+1/3*res_fine, rod_s23+rod_s12+rod_width*1.5-2/3*res_fine, rod_s23+rod_s12+rod_width*2.5-1/3*res_fine, rod_s23+rod_s12+rod_width*2.5+2/3*res_fine,...
                          -rod_s23-rod_s12-rod_s01-rod_width*2.5-1/3*res_fine, -rod_s23-rod_s12-rod_s01-rod_width*2.5+2/3*res_fine, -rod_s23-rod_s12-rod_s01-rod_width*2.5-rodcpl_width+1/3*res_fine, -rod_s23-rod_s12-rod_s01-rod_width*2.5-rodcpl_width-2/3*res_fine,...
                          rod_s23+rod_s12+rod_s01+rod_width*2.5+1/3*res_fine, rod_s23+rod_s12+rod_s01+rod_width*2.5-2/3*res_fine, rod_s23+rod_s12+rod_s01+rod_width*2.5+rodcpl_width-1/3*res_fine, rod_s23+rod_s12+rod_s01+rod_width*2.5+rodcpl_width+2/3*res_fine,...
                          box_l/2+0.003], res_fine, 1.5);
mesh.x = SmoothMeshLines([mesh_start_x, mesh.x, mesh_stop_x], res_coarse, 1.5);
mesh.y = SmoothMeshLines([0, -box_w/2-0.003,...
                          -box_w/2+rod_len-(res_fine*1/3), -box_w/2+rod_len_end+(res_fine*2/3), box_w/2-rod_len_end+(res_fine*1/3), box_w/2-rod_len-(res_fine*2/3),...
                          box_w/2+0.003], res_fine, 1.5);
mesh.y = SmoothMeshLines([mesh_start_y, mesh.y, mesh_stop_y], res_coarse, 1.5);
mesh.z = SmoothMeshLines([-dist_pcb_thick, 0, rod_thick, rod_thick+dist_pcb_thick], res_fine, 1.5);
mesh.z = SmoothMeshLines([mesh_start_z, mesh.z, mesh_stop_z], res_coarse, 1.5);
CSX = DefineRectGrid(CSX, 1, mesh);

#CSX = AddDump(CSX,'Et');
#CSX = AddBox(CSX,'Et',0,[-box_l/2-0.003 mesh_start_y -0.0015],[box_l/2+0.003 mesh_stop_y 0.0025]);

###############################
## PREPARE AND SIMULATE
###############################

#set files, paths, write data, show geometry, run the simulator
confirm_recursive_rmdir(0); #don't want asking when remomoving directory
sim_path = 'qe_sim_path_4ghz';
sim_file = 'sim_file.xml';
rmdir(sim_path, 's');
mkdir(sim_path);
#save structures with data about simulation to file
WriteOpenEMS([sim_path '/' sim_file], FDTD, CSX);
#run a geometry viewer
CSXGeomPlot([sim_path '/' sim_file]);
#simulate
RunOpenEMS(sim_path, sim_file);


###############################
## PROCESS RESULTS
###############################

#S11 plot
f = linspace(fmin, fmax, 10000);
port = calcPort(port, sim_path, f, 'RefImpedance', impedance);
s11 = port{1}.uf.ref ./ port{1}.uf.inc;
s21 = port{2}.uf.ref ./ port{1}.uf.inc;
plot(f/1e9, 20*log10(abs(s11)), 'k-', 'LineWidth', 2);
hold on;
grid on;
plot(f/1e9, 20*log10(abs(s21)), 'r-', 'LineWidth', 2);
legend('S_{11}','S_{21}');
ylabel('S-Parameter (dB)','FontSize',12);
xlabel('frequency (GHz) \rightarrow','FontSize',12);

#results processing
tmp = -1000;
for i = 1:length(f)
  if 20*log10(abs(s21(i))) > tmp
    tmp = 20*log10(abs(s21(i)));
    qe_fpeak = f(i);
  end;
end;
qe_3db_target = tmp - 3;
for i = 1:length(f)
  if 20*log10(abs(s21(i))) > qe_3db_target;
    qe_f3db1 = f(i);
    break;
  end;
end;
for i = length(f):-1:1
  if 20*log10(abs(s21(i))) > qe_3db_target;
    qe_f3db2 = f(i);
    break;
  end;
end;

qe_fpeak
qe = qe_fpeak / (qe_f3db2 - qe_f3db1)

##qe_fpeak =  3996199619.96200
##qe =  53.136
