close all;
clear;
clc

addpath('/opt/openEMS/share/openEMS/matlab');
addpath('/opt/openEMS/share/CSXCAD/matlab');


###############################
## PREPARE SIMULATION MODEL
###############################

#constants
physical_constants; #import physical_constants
impedance = 50; #reference impedance
fmin = 2e9;  #min frquency
fmax = 7e9;  #max frquency
er = 4.5; #FR-4 permittivity

#structure dimensions
rod_len = 17.7/1000;
rod_len_end = 17.7/1000;
rod_width = 3/1000;
rod_thick = 1/1000;
rodcpl_width = 3/1000;
dist_pcb_thick = 1.5/1000;
box_w = 21/1000;
box_l = 58/1000;
box_h = rod_thick+dist_pcb_thick*2;
rod_s01 = 1.4/1000;
rod_s12 = 4/1000;
rod_s23 = 4.4/1000;
probe_dist = 3/1000;

#mesh dimensions
res_coarse = c0 / fmax / 70;
res_fine = res_coarse / 4;
mesh_start_x = -box_l/2;
mesh_stop_x = box_l/2;
mesh_start_y = -box_w/2;
mesh_stop_y = box_w/2;
mesh_start_z = -box_h/2;
mesh_stop_z = box_h/2;

#setup FDTD
FDTD = InitFDTD('EndCriteria', 1e-6);
FDTD = SetGaussExcite(FDTD, (fmax+fmin)/2, (fmax-fmin)/2);  #finite energy excitation
BC = {'MUR' 'MUR' 'PEC' 'PEC' 'PEC' 'PEC'}; #boundary conditions: metal around
FDTD = SetBoundaryCond(FDTD, BC);

#setup CSX
CSX = InitCSX();

#add materials
CSX = AddMetal(CSX,'Metal');  #new metal material

#add resonators
start = [-rod_width/2, -box_w/2, -rod_thick/2];
stop = [rod_width/2, -box_w/2+rod_len, rod_thick/2];
CSX = AddBox(CSX, 'Metal', 400, start, stop);
#
##start = [rod_s23+rod_width*0.5, box_w/2, -rod_thick/2];
##stop = [rod_s23+rod_width*1.5, box_w/2-rod_len, rod_thick/2];
##CSX = AddBox(CSX, 'Metal', 400, start, stop);
start = [-rod_s23-rod_width*0.5, box_w/2, -rod_thick/2];
stop = [-rod_s23-rod_width*1.5, box_w/2-rod_len, rod_thick/2];
CSX = AddBox(CSX, 'Metal', 400, start, stop);
#
##start = [rod_s23+rod_s12+rod_width*1.5, -box_w/2, -rod_thick/2];
##stop = [rod_s23+rod_s12+rod_width*2.5, -box_w/2+rod_len, rod_thick/2];
##CSX = AddBox(CSX, 'Metal', 400, start, stop);
##start = [-rod_s23-rod_s12-rod_width*1.5, -box_w/2, -rod_thick/2];
##stop = [-rod_s23-rod_s12-rod_width*2.5, -box_w/2+rod_len, rod_thick/2];
##CSX = AddBox(CSX, 'Metal', 400, start, stop);
#add connections
start = [-rod_s23-rod_width*1.5-probe_dist-0.0002, 0.0002, -rod_thick/2-dist_pcb_thick+0.001];
stop = [-rod_s23-rod_width*1.5-probe_dist+0.0002, -0.0002, -rod_thick/2-dist_pcb_thick+0.0014];
CSX = AddBox(CSX, 'Metal', 400, start, stop);
start = [rod_width*0.5+probe_dist-0.0002, 0.0002, rod_thick/2+dist_pcb_thick-0.001];
stop = [rod_width*0.5+probe_dist+0.0002, -0.0002, rod_thick/2+dist_pcb_thick-0.0014];
CSX = AddBox(CSX, 'Metal', 400, start, stop);

#add ports
start = [-rod_s23-rod_width*1.5-probe_dist, 0, -rod_thick/2-dist_pcb_thick];
stop = [-rod_s23-rod_width*1.5-probe_dist, 0, -rod_thick/2-dist_pcb_thick+0.001];
[CSX,port{1}] = AddLumpedPort(CSX, 999, 1, impedance, start, stop, [0 0 1], true);
start = [rod_width*0.5+probe_dist, 0, rod_thick/2+dist_pcb_thick];
stop = [rod_width*0.5+probe_dist, 0, rod_thick/2+dist_pcb_thick-0.001];
[CSX,port{2}] = AddLumpedPort(CSX, 999, 2, impedance, start, stop, [0 0 -1]);

setup mesh
mesh.x = SmoothMeshLines([-rod_s23-rod_width*1.5-probe_dist-0.010,...
                          -rod_s23-rod_width*1.5-probe_dist, rod_width*0.5+probe_dist,...
                          -rod_width*0.5+1/3*res_fine, -rod_width*0.5-2/3*res_fine, rod_width*0.5-1/3*res_fine, rod_width*0.5+2/3*res_fine,...
                          -rod_s23-rod_width*0.5-1/3*res_fine, -rod_s23-rod_width*0.5+2/3*res_fine, -rod_s23-rod_width*1.5+1/3*res_fine, -rod_s23-rod_width*1.5-2/3*res_fine,...
                          rod_width*0.5+probe_dist+0.010], res_fine, 1.5);
mesh.y = SmoothMeshLines([mesh_start_y, 0,...
                          -box_w/2+rod_len-(res_fine*1/3), -box_w/2+rod_len+(res_fine*2/3), box_w/2-rod_len+(res_fine*1/3), box_w/2-rod_len-(res_fine*2/3),...
                          mesh_stop_y], res_fine, 1.5);
mesh.z = SmoothMeshLines([mesh_start_z, 0,...
                          -rod_thick/2+1/3*res_fine, -rod_thick/2-2/3*res_fine, rod_thick/2-1/3*res_fine, rod_thick/2+2/3*res_fine,...
                          -rod_thick/2-dist_pcb_thick+0.001, rod_thick/2+dist_pcb_thick-0.001,...
                          mesh_stop_z], res_fine, 1.5);
CSX = DefineRectGrid(CSX, 1, mesh);


###############################
## PREPARE AND SIMULATE
###############################

#set files, paths, write data, show geometry, run the simulator
confirm_recursive_rmdir(0); #don't want asking when remomoving directory
sim_path = 'sim_path_4ghz_m23';
sim_file = 'sim_file.xml';
rmdir(sim_path, 's');
mkdir(sim_path);
#save structures with data about simulation to file
WriteOpenEMS([sim_path '/' sim_file], FDTD, CSX);
#run a geometry viewer
CSXGeomPlot([sim_path '/' sim_file]);
#simulate
RunOpenEMS(sim_path, sim_file);


###############################
## PROCESS RESULTS
###############################

#S11 plot
f = linspace(fmin, fmax, 2000);
port = calcPort(port, sim_path, f, 'RefImpedance', impedance);
s11 = port{1}.uf.ref ./ port{1}.uf.inc;
s21 = port{2}.uf.ref ./ port{1}.uf.inc;
plot(f/1e9, 20*log10(abs(s11)), 'k-', 'LineWidth', 2);
hold on;
grid on;
plot(f/1e9, 20*log10(abs(s21)), 'r-', 'LineWidth', 2);
legend('S_{11}','S_{21}');
ylabel('S-Parameter (dB)','FontSize',12);
xlabel('frequency (GHz) \rightarrow','FontSize',12);
title('m23');
#results processing
tmp = -1000;
for i = 1:length(f)
  if 20*log10(abs(s21(i))) > tmp
    tmp = 20*log10(abs(s21(i)));
    m23_fpeak1 = f(i);
  end;
  if ( (tmp-3 > 20*log10(abs(s21(i)))) && (20*log10(abs(s21(i))) > -60) )
    break;
  end;
end;
tmp = -1000;
for i = length(f)/2:-1:1
  if 20*log10(abs(s21(i))) > tmp
    tmp = 20*log10(abs(s21(i)));
    m23_fpeak2 = f(i);
  end;
  if ( (tmp-3 > 20*log10(abs(s21(i)))) && (20*log10(abs(s21(i))) > -60) )
    break;
  end;
end;

m23_f0 = sqrt(m23_fpeak2*m23_fpeak1)
m23 = (m23_fpeak2 - m23_fpeak1) / m23_f0

##m23_f0 =  4032163535
##m23 =  0.014267
